@@ -5,12 +5,12 @@ use strict;
 use warnings;
 
 sub description { 'GMX: https://www.gmx.net' }
-sub make {
+sub inquire {
     # Detect an error from GMX and mail.com
     # @param    [Hash] mhead    Message headers of a bounce email
     # @param    [String] mbody  Message body of a bounce email
     # @return   [Hash]          Bounce data list and message/rfc822 part
-    # @return   [Undef]         failed to parse or the arguments are missing
+    # @return   [undef]         failed to parse or the arguments are missing
     # @since v4.1.4
     my $class = shift;
     my $mhead = shift // return undef;
@@ -22,20 +22,21 @@ sub make {
     # X-UI-Out-Filterresults: unknown:0;
     return undef unless defined $mhead->{'x-gmx-antispam'};
 
+    require Sisimai::SMTP::Command;
     state $indicators = __PACKAGE__->INDICATORS;
-    state $rebackbone = qr|^---[ ]The[ ]header[ ]of[ ]the[ ]original[ ]message[ ]is[ ]following[.][ ]---|m;
+    state $boundaries = ['--- The header of the original message is following. ---'];
     state $startingof = { 'message' => ['This message was created automatically by mail delivery software'] };
     state $messagesof = { 'expired' => ['delivery retry timeout exceeded'] };
 
     my $dscontents = [__PACKAGE__->DELIVERYSTATUS];
-    my $emailsteak = Sisimai::RFC5322->fillet($mbody, $rebackbone);
+    my $emailparts = Sisimai::RFC5322->part($mbody, $boundaries);
     my $readcursor = 0;     # (Integer) Points the current cursor position
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
     my $v = undef;
 
-    for my $e ( split("\n", $emailsteak->[0]) ) {
-        # Read error messages and delivery status lines from the head of the email
-        # to the previous line of the beginning of the original message.
+    for my $e ( split("\n", $emailparts->[0]) ) {
+        # Read error messages and delivery status lines from the head of the email to the previous
+        # line of the beginning of the original message.
         unless( $readcursor ) {
             # Beginning of the bounce message or message/delivery-status part
             $readcursor |= $indicators->{'deliverystatus'} if index($e, $startingof->{'message'}->[0]) == 0;
@@ -56,7 +57,7 @@ sub make {
         # 5.1.1 <shironeko@example.jp>... User Unknown
         $v = $dscontents->[-1];
 
-        if( $e =~ /\A["]([^ ]+[@][^ ]+)["]:\z/ || $e =~ /\A[<]([^ ]+[@][^ ]+)[>]\z/ ) {
+        if( index($e, '@') > 1 && (index($e, '"') == 0 || index($e, '<') == 0) ) {
             # "shironeko@example.jp":
             # ---- OR ----
             # <kijitora@6jo.example.co.jp>
@@ -68,20 +69,21 @@ sub make {
                 push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
                 $v = $dscontents->[-1];
             }
-            $v->{'recipient'} = $1;
+            $v->{'recipient'} = Sisimai::Address->s3s4($e);
             $recipients++;
 
-        } elsif( $e =~ /\ASMTP error .+ ([A-Z]{4}) command:\z/ ) {
+        } elsif( index($e, 'SMTP error ') == 0 ) {
             # SMTP error from remote server after RCPT command:
-            $v->{'command'} = $1;
+            $v->{'command'} = Sisimai::SMTP::Command->find($e);
 
-        } elsif( $e =~ /\Ahost:[ \t]*(.+)\z/ ) {
+        } elsif( index($e, 'host: ') == 0 ) {
             # host: mx.example.jp
-            $v->{'rhost'} = $1;
+            $v->{'rhost'} = substr($e, 6, );
 
         } else {
             # Get error message
-            if( $e =~ /\b[45][.]\d[.]\d\b/ || $e =~ /[<][^ ]+[@][^ ]+[>]/ || $e =~ /\b[45]\d{2}\b/ ) {
+            if( Sisimai::SMTP::Status->find($e) || Sisimai::String->aligned(\$e, ['<', '@', '>']) ) {
+                # 5.1.1 <shironeko@example.jp>... User Unknown
                 $v->{'diagnosis'} ||= $e;
 
             } else {
@@ -105,12 +107,12 @@ sub make {
 
         SESSION: for my $r ( keys %$messagesof ) {
             # Verify each regular expression of session errors
-            next unless grep { index($e->{'diagnosis'}, $_) > -1 } @{ $messagesof->{ $r } };
+            next unless grep { index($e->{'diagnosis'}, $_) > -1 } $messagesof->{ $r }->@*;
             $e->{'reason'} = $r;
             last;
         }
     }
-    return { 'ds' => $dscontents, 'rfc822' => $emailsteak->[1] };
+    return { 'ds' => $dscontents, 'rfc822' => $emailparts->[1] };
 }
 
 1;
@@ -128,8 +130,8 @@ Sisimai::Lhost::GMX - bounce mail parser class for C<GMX> and mail.com.
 
 =head1 DESCRIPTION
 
-Sisimai::Lhost::GMX parses a bounce email which created by C<GMX>.
-Methods in the module are called from only Sisimai::Message.
+Sisimai::Lhost::GMX parses a bounce email which created by C<GMX>. Methods in the module are called
+from only Sisimai::Message.
 
 =head1 CLASS METHODS
 
@@ -139,10 +141,10 @@ C<description()> returns description string of this module.
 
     print Sisimai::Lhost::GMX->description;
 
-=head2 C<B<make(I<header data>, I<reference to body string>)>>
+=head2 C<B<inquire(I<header data>, I<reference to body string>)>>
 
-C<make()> method parses a bounced email and return results as a array reference.
-See Sisimai::Message for more details.
+C<inquire()> method parses a bounced email and return results as a array reference. See Sisimai::Message
+for more details.
 
 =head1 AUTHOR
 
@@ -150,7 +152,7 @@ azumakuniyuki
 
 =head1 COPYRIGHT
 
-Copyright (C) 2014-2020 azumakuniyuki, All rights reserved.
+Copyright (C) 2014-2023 azumakuniyuki, All rights reserved.
 
 =head1 LICENSE
 
