@@ -5,12 +5,12 @@ use strict;
 use warnings;
 
 sub description { 'IPSWITCH IMail Server' }
-sub make {
+sub inquire {
     # Detect an error from IMailServer
     # @param    [Hash] mhead    Message headers of a bounce email
     # @param    [String] mbody  Message body of a bounce email
     # @return   [Hash]          Bounce data list and message/rfc822 part
-    # @return   [Undef]         failed to parse or the arguments are missing
+    # @return   [undef]         failed to parse or the arguments are missing
     # @since v4.1.1
     my $class = shift;
     my $mhead = shift // return undef;
@@ -18,61 +18,55 @@ sub make {
     my $match = 0;
 
     # X-Mailer: <SMTP32 v8.22>
-    $match ||= 1 if $mhead->{'subject'} =~ /\AUndeliverable Mail[ ]*\z/;
+    $match ||= 1 if index($mhead->{'subject'}, 'Undeliverable Mail ') == 0;
     $match ||= 1 if defined $mhead->{'x-mailer'} && index($mhead->{'x-mailer'}, '<SMTP32 v') == 0;
     return undef unless $match;
 
-    state $rebackbone = qr|^Original[ ]message[ ]follows[.]|m;
+    state $boundaries = ['Original message follows.'];
     state $startingof = { 'error' => ['Body of message generated response:'] };
-    state $recommands = {
-        'conn' => qr/(?:SMTP connection failed,|Unexpected connection response from server:)/,
-        'ehlo' => qr|Unexpected response to EHLO/HELO:|,
-        'mail' => qr|Server response to MAIL FROM:|,
-        'rcpt' => qr|Additional RCPT TO generated following response:|,
-        'data' => qr|DATA command generated response:|,
-    };
     state $refailures = {
-        'hostunknown'   => qr/Unknown host/,
-        'userunknown'   => qr/\A(?:Unknown user|Invalid final delivery userid)/,
-        'mailboxfull'   => qr/\AUser mailbox exceeds allowed size/,
-        'securityerror' => qr/\ARequested action not taken: virus detected/,
-        'undefined'     => qr/\Aundeliverable to/,
-        'expired'       => qr/\ADelivery failed \d+ attempts/,
+        'hostunknown'   => ['Unknown host'],
+        'userunknown'   => ['Unknown user', 'Invalid final delivery userid'],
+        'mailboxfull'   => ['User mailbox exceeds allowed size'],
+        'virusdetected' => ['Requested action not taken: virus detected'],
+        'undefined'     => ['undeliverable to'],
+        'expired'       => ['Delivery failed '],
     };
 
     my $dscontents = [__PACKAGE__->DELIVERYSTATUS];
-    my $emailsteak = Sisimai::RFC5322->fillet($mbody, $rebackbone);
+    my $emailparts = Sisimai::RFC5322->part($mbody, $boundaries);
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
     my $v = undef;
 
-    for my $e ( split("\n", $emailsteak->[0]) ) {
-        # Read error messages and delivery status lines from the head of the email
-        # to the previous line of the beginning of the original message.
+    for my $e ( split("\n", $emailparts->[0]) ) {
+        # Read error messages and delivery status lines from the head of the email to the previous
+        # line of the beginning of the original message.
 
         # Unknown user: kijitora@example.com
         #
         # Original message follows.
         $v = $dscontents->[-1];
 
-        if( $e =~ /\A([^ ]+)[ ](.+)[:][ \t]*([^ ]+[@][^ ]+)/ ) {
+        my $p0 = index($e, ': ');
+        if( $p0 > 8 && Sisimai::String->aligned(\$e, [': ', '@']) ) {
             # Unknown user: kijitora@example.com
             if( $v->{'recipient'} ) {
                 # There are multiple recipient addresses in the message body.
                 push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
                 $v = $dscontents->[-1];
             }
-            $v->{'diagnosis'} = $1.' '.$2;
-            $v->{'recipient'} = $3;
+            $v->{'diagnosis'} = $e;
+            $v->{'recipient'} = Sisimai::Address->s3s4(substr($e, $p0 + 2));
             $recipients++;
 
-        } elsif( $e =~ /\Aundeliverable[ ]+to[ ]+(.+)\z/ ) {
+        } elsif( index($e, 'undeliverable ') == 0 ) {
             # undeliverable to kijitora@example.com
             if( $v->{'recipient'} ) {
                 # There are multiple recipient addresses in the message body.
                 push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
                 $v = $dscontents->[-1];
             }
-            $v->{'recipient'} = Sisimai::Address->s3s4($1);
+            $v->{'recipient'} = Sisimai::Address->s3s4($e);
             $recipients++;
 
         } else {
@@ -84,6 +78,7 @@ sub make {
     }
     return undef unless $recipients;
 
+    require Sisimai::SMTP::Command;
     for my $e ( @$dscontents ) {
         if( exists $e->{'alterrors'} && $e->{'alterrors'} ) {
             # Copy alternative error message
@@ -92,22 +87,16 @@ sub make {
             delete $e->{'alterrors'};
         }
         $e->{'diagnosis'} = Sisimai::String->sweep($e->{'diagnosis'});
-
-        COMMAND: for my $r ( keys %$recommands ) {
-            # Detect SMTP command from the message
-            next unless $e->{'diagnosis'} =~ $recommands->{ $r };
-            $e->{'command'} = uc $r;
-            last;
-        }
+        $e->{'command'}   = Sisimai::SMTP::Command->find($e->{'diagnosis'});
 
         SESSION: for my $r ( keys %$refailures ) {
             # Verify each regular expression of session errors
-            next unless $e->{'diagnosis'} =~ $refailures->{ $r };
+            next unless grep { index($e->{'diagnosis'}, $_) > -1 } $refailures->{ $r }->@*;
             $e->{'reason'} = $r;
             last;
         }
     }
-    return { 'ds' => $dscontents, 'rfc822' => $emailsteak->[1] };
+    return { 'ds' => $dscontents, 'rfc822' => $emailparts->[1] };
 }
 
 1;
@@ -125,9 +114,8 @@ Sisimai::Lhost::IMailServer - bounce mail parser class for C<IMail Server>.
 
 =head1 DESCRIPTION
 
-Sisimai::Lhost::IMailServer parses a bounce email which created by
-C<Ipswitch IMail Server>. Methods in the module are called from only
-Sisimai::Message.
+Sisimai::Lhost::IMailServer parses a bounce email which created by C<Ipswitch IMail Server>. Methods
+in the module are called from only Sisimai::Message.
 
 =head1 CLASS METHODS
 
@@ -137,10 +125,10 @@ C<description()> returns description string of this module.
 
     print Sisimai::Lhost::IMailServer->description;
 
-=head2 C<B<make(I<header data>, I<reference to body string>)>>
+=head2 C<B<inquire(I<header data>, I<reference to body string>)>>
 
-C<make()> method parses a bounced email and return results as a array reference.
-See Sisimai::Message for more details.
+C<inquire()> method parses a bounced email and return results as a array reference. See Sisimai::Message
+for more details.
 
 =head1 AUTHOR
 
@@ -148,7 +136,7 @@ azumakuniyuki
 
 =head1 COPYRIGHT
 
-Copyright (C) 2014-2021 azumakuniyuki, All rights reserved.
+Copyright (C) 2014-2023 azumakuniyuki, All rights reserved.
 
 =head1 LICENSE
 
