@@ -5,12 +5,12 @@ use strict;
 use warnings;
 
 sub description { 'Trend Micro InterScan Messaging Security Suite' }
-sub make {
+sub inquire {
     # Detect an error from InterScanMSS
     # @param    [Hash] mhead    Message headers of a bounce email
     # @param    [String] mbody  Message body of a bounce email
     # @return   [Hash]          Bounce data list and message/rfc822 part
-    # @return   [Undef]         failed to parse or the arguments are missing
+    # @return   [undef]         failed to parse or the arguments are missing
     # @since v4.1.2
     my $class = shift;
     my $mhead = shift // return undef;
@@ -28,48 +28,52 @@ sub make {
     $match ||= 1 if grep { $mhead->{'subject'} eq $_ } @$tryto;
     return undef unless $match;
 
-    state $rebackbone = qr|^Content-type:[ ]message/rfc822|m;
+    require Sisimai::SMTP::Command;
+    state $boundaries = ['Content-type: message/rfc822'];
+
     my $dscontents = [__PACKAGE__->DELIVERYSTATUS];
-    my $emailsteak = Sisimai::RFC5322->fillet($mbody, $rebackbone);
+    my $emailparts = Sisimai::RFC5322->part($mbody, $boundaries);
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
     my $v = undef;
 
-    for my $e ( split("\n", $emailsteak->[0]) ) {
-        # Read error messages and delivery status lines from the head of the email
-        # to the previous line of the beginning of the original message.
+    for my $e ( split("\n", $emailparts->[0]) ) {
+        # Read error messages and delivery status lines from the head of the email to the previous
+        # line of the beginning of the original message.
         next unless length $e;
 
         $v = $dscontents->[-1];
-        if( $e =~ /\A.+[<>]{3}[ \t]+.+[<]([^ ]+[@][^ ]+)[>]\z/ ||
-            $e =~ /\A.+[<>]{3}[ \t]+.+[<]([^ ]+[@][^ ]+)[>]/   ||
-            $e =~ /\A(?:Reason:[ ]+)?Unable[ ]to[ ]deliver[ ]message[ ]to[ ][<](.+)[>]/ ) {
+        my $p1 = index($e, ' <<< ');    # Sent <<< ...
+        my $p2 = index($e, ' >>> ');    # Received >>> ...
+        if( index($e, '@') > 1 && index($e, ' <') > 1 && ($p1 > 1 || $p2 > 1 || index($e, 'Unable to deliver ') > -1) ) {
             # Sent <<< RCPT TO:<kijitora@example.co.jp>
             # Received >>> 550 5.1.1 <kijitora@example.co.jp>... user unknown
+            # Received >>> 550 5.1.1 unknown user.
             # Unable to deliver message to <kijitora@neko.example.jp>
-            my $cr = $1;
+            # Unable to deliver message to <neko@example.jp> (and other recipients in the same domain).
+            my $cr = substr($e, rindex($e, '<') + 1, rindex($e, '>') - rindex($e, '<') - 1);
             if( $v->{'recipient'} && $cr ne $v->{'recipient'} ) {
                 # There are multiple recipient addresses in the message body.
                 push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
                 $v = $dscontents->[-1];
             }
             $v->{'recipient'} = $cr;
-            $v->{'diagnosis'} = $e if $e =~ /Unable[ ]to[ ]deliver[ ]/;
+            $v->{'diagnosis'} = $e if index($e, 'Unable to deliver ') > -1;
             $recipients = scalar @$dscontents;
         }
 
-        if( $e =~ /\ASent[ \t]+[<]{3}[ \t]+([A-Z]{4})[ \t]/ ) {
+        if( index($e, 'Sent <<< ') == 0 ) {
             # Sent <<< RCPT TO:<kijitora@example.co.jp>
-            $v->{'command'} = $1
+            $v->{'command'} = Sisimai::SMTP::Command->find($e);
 
-        } elsif( $e =~ /\AReceived[ \t]+[>]{3}[ \t]+(\d{3}[ \t]+.+)\z/ ) {
+        } elsif( index($e, 'Received >>> ') == 0 ) {
             # Received >>> 550 5.1.1 <kijitora@example.co.jp>... user unknown
-            $v->{'diagnosis'} = $1;
+            $v->{'diagnosis'} = substr($e, index($e, ' >>> ') + 4, );
 
-        } else {
+        } elsif( $p1 > 0 || $p2 > 0 ) {
             # Error message in non-English
-            next unless $e =~ /[ ][<>]{3}[ ]/;
-            $v->{'command'}   = $1 if $e =~ /[ ][>]{3}[ ]([A-Z]{4})/; # >>> RCPT TO ...
-            $v->{'diagnosis'} = $1 if $e =~ /[ ][<]{3}[ ](.+)/;       # <<< 550 5.1.1 User unknown
+            $v->{'command'} = Sisimai::SMTP::Command->find($e) if index($e, ' >>> ') > -1;
+            my $p3 = index($e, ' <<< '); next if $p3 == -1;
+            $v->{'diagnosis'} = substr($e, $p3 + 4,);
         }
     }
     return undef unless $recipients;
@@ -77,9 +81,9 @@ sub make {
     for my $e ( @$dscontents ) {
         # Set default values if each value is empty.
         $e->{'diagnosis'} = Sisimai::String->sweep($e->{'diagnosis'});
-        $e->{'reason'} = 'userunknown' if $e->{'diagnosis'} =~ /Unable[ ]to[ ]deliver/;
+        $e->{'reason'} = 'userunknown' if index($e->{'diagnosis'}, 'Unable to deliver') > -1;
     }
-    return { 'ds' => $dscontents, 'rfc822' => $emailsteak->[1] };
+    return { 'ds' => $dscontents, 'rfc822' => $emailparts->[1] };
 }
 
 1;
@@ -89,8 +93,7 @@ __END__
 
 =head1 NAME
 
-Sisimai::Lhost::InterScanMSS - bounce mail parser class for
-C<Trend Micro InterScan Messaging Security Suite>.
+Sisimai::Lhost::InterScanMSS - bounce mail parser class for C<Trend Micro InterScan Messaging Security Suite>.
 
 =head1 SYNOPSIS
 
@@ -98,9 +101,8 @@ C<Trend Micro InterScan Messaging Security Suite>.
 
 =head1 DESCRIPTION
 
-Sisimai::Lhost::InterScanMSS parses a bounce email which created by C<Trend Micro
-InterScan Messaging Security Suite>. Methods in the module are called from only
-Sisimai::Message.
+Sisimai::Lhost::InterScanMSS parses a bounce email which created by C<Trend Micro InterScan Messaging Security Suite>.
+Methods in the module are called from only Sisimai::Message.
 
 =head1 CLASS METHODS
 
@@ -110,10 +112,10 @@ C<description()> returns description string of this module.
 
     print Sisimai::Lhost::InterScanMSS->description;
 
-=head2 C<B<make(I<header data>, I<reference to body string>)>>
+=head2 C<B<inquire(I<header data>, I<reference to body string>)>>
 
-C<make()> method parses a bounced email and return results as a array reference.
-See Sisimai::Message for more details.
+C<inquire()> method parses a bounced email and return results as a array reference. See Sisimai::Message
+for more details.
 
 =head1 AUTHOR
 
@@ -121,7 +123,7 @@ azumakuniyuki
 
 =head1 COPYRIGHT
 
-Copyright (C) 2014-2020 azumakuniyuki, All rights reserved.
+Copyright (C) 2014-2021,2023 azumakuniyuki, All rights reserved.
 
 =head1 LICENSE
 
