@@ -4,44 +4,43 @@ use feature ':5.10';
 use strict;
 use warnings;
 
-state $Indicators = __PACKAGE__->INDICATORS;
-state $ReBackbone = qr<^(?:Content-type:[ \t]*message/rfc822|Return-path:[ \t]*)>m;
-state $StartingOf = { 'message' => ['This report relates to a message you sent with the following header fields:'] };
-state $MessagesOf = { 'hostunknown' => ['Illegal host/domain name found'] };
-
 sub description { 'Oracle Communications Messaging Server' }
-sub make {
+sub inquire {
     # Detect an error from Oracle Communications Messaging Server
     # @param    [Hash] mhead    Message headers of a bounce email
     # @param    [String] mbody  Message body of a bounce email
     # @return   [Hash]          Bounce data list and message/rfc822 part
-    # @return   [Undef]         failed to parse or the arguments are missing
+    # @return   [undef]         failed to parse or the arguments are missing
     # @since v4.1.3
     my $class = shift;
     my $mhead = shift // return undef;
     my $mbody = shift // return undef;
     my $match = 0;
 
-    # 'received' => qr/[ ][(]MessagingServer[)][ ]with[ ]/,
     $match ||= 1 if rindex($mhead->{'content-type'}, 'Boundary_(ID_') > -1;
     $match ||= 1 if index($mhead->{'subject'}, 'Delivery Notification: ') == 0;
     return undef unless $match;
 
+    state $indicators = __PACKAGE__->INDICATORS;
+    state $boundaries = ['Content-Type: message/rfc822', 'Return-path: '];
+    state $startingof = { 'message' => ['This report relates to a message you sent with the following header fields:'] };
+    state $messagesof = { 'hostunknown' => ['Illegal host/domain name found'] };
+
     my $dscontents = [__PACKAGE__->DELIVERYSTATUS];
-    my $emailsteak = Sisimai::RFC5322->fillet($mbody, $ReBackbone);
+    my $emailparts = Sisimai::RFC5322->part($mbody, $boundaries);
     my $readcursor = 0;     # (Integer) Points the current cursor position
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
     my $v = undef;
 
-    for my $e ( split("\n", $emailsteak->[0]) ) {
-        # Read error messages and delivery status lines from the head of the email
-        # to the previous line of the beginning of the original message.
+    for my $e ( split("\n", $emailparts->[0]) ) {
+        # Read error messages and delivery status lines from the head of the email to the previous
+        # line of the beginning of the original message.
         unless( $readcursor ) {
             # Beginning of the bounce message or message/delivery-status part
-            $readcursor |= $Indicators->{'deliverystatus'} if index($e, $StartingOf->{'message'}->[0]) == 0;
+            $readcursor |= $indicators->{'deliverystatus'} if index($e, $startingof->{'message'}->[0]) == 0;
             next;
         }
-        next unless $readcursor & $Indicators->{'deliverystatus'};
+        next unless $readcursor & $indicators->{'deliverystatus'};
         next unless length $e;
 
         # --Boundary_(ID_0000000000000000000000)
@@ -64,45 +63,50 @@ sub make {
         #   Remote system: dns;mx.example.jp (TCP|17.111.174.67|47323|192.0.2.225|25) (6jo.example.jp ESMTP SENDMAIL-VM)
         $v = $dscontents->[-1];
 
-        if( $e =~ /\A[ \t]+Recipient address:[ \t]*([^ ]+[@][^ ]+)\z/ ) {
+        if( index($e, '  Recipient address: ') == 0 && index($e, '@') > 1 ) {
             #   Recipient address: kijitora@example.jp
             if( $v->{'recipient'} ) {
                 # There are multiple recipient addresses in the message body.
                 push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
                 $v = $dscontents->[-1];
             }
-            $v->{'recipient'} = Sisimai::Address->s3s4($1);
+            $v->{'recipient'} = Sisimai::Address->s3s4(substr($e, rindex($e, ' ') + 1),);
             $recipients++;
 
-        } elsif( $e =~ /\A[ \t]+Original address:[ \t]*([^ ]+[@][^ ]+)\z/ ) {
+        } elsif( index($e, '  Original address: ') == 0 && index($e, '@') > 1 ) {
             #   Original address: kijitora@example.jp
-            $v->{'recipient'} = Sisimai::Address->s3s4($1);
+            $v->{'recipient'} = Sisimai::Address->s3s4(substr($e, rindex($e, ' ') + 1),);
 
-        } elsif( $e =~ /\A[ \t]+Date:[ \t]*(.+)\z/ ) {
+        } elsif( index($e, '  Date: ') == 0 ) {
             #   Date: Fri, 21 Nov 2014 23:34:45 +0900
-            $v->{'date'} = $1;
+            $v->{'date'} = substr($e, index($e, ':') + 2,);
 
-        } elsif( $e =~ /\A[ \t]+Reason:[ \t]*(.+)\z/ ) {
+        } elsif( index($e, '  Reason: ') == 0 ) {
             #   Reason: Remote SMTP server has rejected address
-            $v->{'diagnosis'} = $1;
+            $v->{'diagnosis'} = substr($e, index($e, ':') + 2,);
 
-        } elsif( $e =~ /\A[ \t]+Diagnostic code:[ \t]*([^ ]+);(.+)\z/ ) {
+        } elsif( index($e, '  Diagnostic code: ') == 0 ) {
             #   Diagnostic code: smtp;550 5.1.1 <kijitora@example.jp>... User Unknown
-            $v->{'spec'} = uc $1;
-            $v->{'diagnosis'} = $2;
+            my $p1 = index($e, ':');
+            my $p2 = index($e, ';');
+            $v->{'spec'} = uc substr($e, $p1 + 2, $p2 - $p1 - 2);
+            $v->{'diagnosis'} = substr($e, $p2 + 1,);
 
-        } elsif( $e =~ /\A[ \t]+Remote system:[ \t]*dns;([^ ]+)[ \t]*([^ ]+)[ \t]*.+\z/ ) {
+        } elsif( index($e, '  Remote system: ') == 0 ) {
             #   Remote system: dns;mx.example.jp (TCP|17.111.174.67|47323|192.0.2.225|25)
             #     (6jo.example.jp ESMTP SENDMAIL-VM)
-            my $remotehost = $1; # remote host
-            my $sessionlog = $2; # smtp session
+            my $p1 = index($e, ';');
+            my $p2 = index($e, '(');
+
+            my $remotehost = substr($e, $p1 + 1, $p2 - $p1 - 2);
+            my $sessionlog = [split('|', substr($e, $p2,))];
             $v->{'rhost'} = $remotehost;
 
             # The value does not include ".", use IP address instead.
             # (TCP|17.111.174.67|47323|192.0.2.225|25)
-            next unless $sessionlog =~ /\A[(]TCP|(.+)|\d+|(.+)|\d+[)]/;
-            $v->{'lhost'} = $1;
-            $v->{'rhost'} = $2 unless $remotehost =~ /[^.]+[.][^.]+/;
+            next unless $sessionlog->[0] eq '(TCP';
+            $v->{'lhost'} = $sessionlog->[1];
+            $v->{'rhost'} = $sessionlog->[3] unless index($remotehost, '.') > 1;
 
         } else {
             # Original-envelope-id: 0NFC009FLKOUVMA0@mr21p30im-asmtp004.me.com
@@ -117,20 +121,22 @@ sub make {
             #  (6jo.example.jp ESMTP SENDMAIL-VM)
             # Diagnostic-code: smtp;550 5.1.1 <kijitora@example.jp>... User Unknown
             #
-            if( $e =~ /\AStatus:[ \t]*(\d[.]\d[.]\d)[ \t]*[(](.+)[)]\z/ ) {
+            if( index($e, 'Status: ') == 0 ) {
                 # Status: 5.1.1 (Remote SMTP server has rejected address)
-                $v->{'status'} = $1;
-                $v->{'diagnosis'} ||= $2;
+                my $p1 = index($e, ':');
+                my $p2 = index($e, '(');
+                $v->{'status'}      = substr($e, $p1 + 2, $p2 - $p1 - 3);
+                $v->{'diagnosis'} ||= substr($e, $p2 + 1, index($e, ')') - $p2 - 1);
 
-            } elsif( $e =~ /\AArrival-Date:[ ]*(.+)\z/ ) {
+            } elsif( index($e, 'Arrival-Date: ') == 0 ) {
                 # Arrival-date: Thu, 29 Apr 2014 23:34:45 +0000 (GMT)
-                $v->{'date'} ||= $1;
+                $v->{'date'} ||= substr($e, index($e, ':') + 2,);
 
-            } elsif( $e =~ /\AReporting-MTA:[ ]*(?:DNS|dns);[ ]*(.+)\z/ ) {
+            } elsif( index($e, 'Reporting-MTA: ') == 0 ) {
                 # Reporting-MTA: dns;mr21p30im-asmtp004.me.com (tcp-daemon)
-                my $localhost = $1;
+                my $localhost = substr($e, index($e, ';') + 1,);
                 $v->{'lhost'} ||= $localhost;
-                $v->{'lhost'}   = $localhost unless $v->{'lhost'} =~ /[^.]+[.][^ ]+/;
+                $v->{'lhost'}   = $localhost unless index($v->{'lhost'}, '.') > 0;
             }
         } # End of error message part
     }
@@ -139,14 +145,14 @@ sub make {
     for my $e ( @$dscontents ) {
         $e->{'diagnosis'} = Sisimai::String->sweep($e->{'diagnosis'});
 
-        SESSION: for my $r ( keys %$MessagesOf ) {
+        SESSION: for my $r ( keys %$messagesof ) {
             # Verify each regular expression of session errors
-            next unless grep { index($e->{'diagnosis'}, $_) > -1 } @{ $MessagesOf->{ $r } };
+            next unless grep { index($e->{'diagnosis'}, $_) > -1 } $messagesof->{ $r }->@*;
             $e->{'reason'} = $r;
             last;
         }
     }
-    return { 'ds' => $dscontents, 'rfc822' => $emailsteak->[1] };
+    return { 'ds' => $dscontents, 'rfc822' => $emailparts->[1] };
 }
 
 1;
@@ -156,8 +162,8 @@ __END__
 
 =head1 NAME
 
-Sisimai::Lhost::MessagingServer - bounce mail parser class for
-C<Sun Java System Messaging Server> and C<Oracle Communications Messaging Server>.
+Sisimai::Lhost::MessagingServer - bounce mail parser class for C<Sun Java System Messaging Server>
+and C<Oracle Communications Messaging Server>.
 
 =head1 SYNOPSIS
 
@@ -165,9 +171,8 @@ C<Sun Java System Messaging Server> and C<Oracle Communications Messaging Server
 
 =head1 DESCRIPTION
 
-Sisimai::Lhost::MessagingServer parses a bounce email which created by
-C<Oracle Communications Messaging Server> and C<Sun Java System Messaging Server>.
-Methods in the module are called from only Sisimai::Message.
+Sisimai::Lhost::MessagingServer parses a bounce email which created by C<Oracle Communications Messaging Server>
+and C<Sun Java System Messaging Server>. Methods in the module are called from only Sisimai::Message.
 
 =head1 CLASS METHODS
 
@@ -177,10 +182,10 @@ C<description()> returns description string of this module.
 
     print Sisimai::Lhost::MessagingServer->description;
 
-=head2 C<B<make(I<header data>, I<reference to body string>)>>
+=head2 C<B<inquire(I<header data>, I<reference to body string>)>>
 
-C<make()> method parses a bounced email and return results as a array reference.
-See Sisimai::Message for more details.
+C<inquire()> method parses a bounced email and return results as a array reference. See Sisimai::Message
+for more details.
 
 =head1 AUTHOR
 
@@ -188,7 +193,7 @@ azumakuniyuki
 
 =head1 COPYRIGHT
 
-Copyright (C) 2014-2020 azumakuniyuki, All rights reserved.
+Copyright (C) 2014-2023 azumakuniyuki, All rights reserved.
 
 =head1 LICENSE
 
