@@ -5,18 +5,18 @@ use strict;
 use warnings;
 
 sub description { 'Microsoft Office 365: https://office.microsoft.com/' }
-sub make {
+sub inquire {
     # Detect an error from Microsoft Office 365
     # @param    [Hash] mhead    Message headers of a bounce email
     # @param    [String] mbody  Message body of a bounce email
     # @return   [Hash]          Bounce data list and message/rfc822 part
-    # @return   [Undef]         failed to parse or the arguments are missing
+    # @return   [undef]         failed to parse or the arguments are missing
     # @since v4.1.3
     my $class = shift;
     my $mhead = shift // return undef;
     my $mbody = shift // return undef;
     my $match = 0;
-    my $tryto = qr/.+[.](?:outbound[.]protection|prod)[.]outlook[.]com\b/;
+    my $tryto = ['.outbound.protection.outlook.com', '.prod.outlook.com'];
 
     # X-MS-Exchange-Message-Is-Ndr:
     # X-Microsoft-Antispam-PRVS: <....@...outlook.com>
@@ -26,6 +26,8 @@ sub make {
     # X-MS-Exchange-CrossTenant-FromEntityHeader: Hosted
     # X-MS-Exchange-Transport-CrossTenantHeadersStamped: ...
     $match++ if index($mhead->{'subject'}, 'Undeliverable:') > -1;
+    $match++ if index($mhead->{'subject'}, 'Onbestelbaar:')  > -1;
+    $match++ if index($mhead->{'subject'}, 'Não_entregue:')  > -1;
     $match++ if $mhead->{'x-ms-exchange-message-is-ndr'};
     $match++ if $mhead->{'x-microsoft-antispam-prvs'};
     $match++ if $mhead->{'x-exchange-antispam-report-test'};
@@ -33,31 +35,49 @@ sub make {
     $match++ if $mhead->{'x-ms-exchange-crosstenant-originalarrivaltime'};
     $match++ if $mhead->{'x-ms-exchange-crosstenant-fromentityheader'};
     $match++ if $mhead->{'x-ms-exchange-transport-crosstenantheadersstamped'};
-    $match++ if grep { $_ =~ $tryto } @{ $mhead->{'received'} };
+    $match++ if grep { index($_, $tryto->[0]) > 0 || index($_, $tryto->[1]) > 0 } $mhead->{'received'}->@*;
     if( defined $mhead->{'message-id'} ) {
         # Message-ID: <00000000-0000-0000-0000-000000000000@*.*.prod.outlook.com>
-        $match++ if $mhead->{'message-id'} =~ $tryto;
+        $match++ if grep { index($mhead->{'message-id'}, $_) > 0 } @$tryto;
     }
     return undef if $match < 2;
 
     state $indicators = __PACKAGE__->INDICATORS;
-    state $rebackbone = qr|^Content-Type:[ ]message/rfc822|m;
-    state $markingsof = {
-        'eoe'     => qr/\A(?:Original[ ][Mm]essage[ ][Hh]eaders:?|Message[ ]Hops)/,
-        'error'   => qr/\A(?:Diagnostic[ ]information[ ]for[ ]administrators:|Error[ ]Details)/,
-        'message' => qr{\A(?:
-             Delivery[ ]has[ ]failed[ ]to[ ]these[ ]recipients[ ]or[ ]groups:
-            |Original[ ]Message[ ]Details
-            |.+[ ]rejected[ ]your[ ]message[ ]to[ ]the[ ]following[ ]e[-]?mail[ ]addresses:
-            )
-        }x,
+    state $boundaries = ['Content-Type: message/rfc822', 'Original message headers:'];
+    state $commandset = { 'RCPT' => ['unknown recipient or mailbox unavailable ->', '@'] };
+    state $startingof = {
+        'eoe'     => [
+            'Original message headers:', 'Original Message Headers:',
+            'Message Hops',
+            'alhos originais da mensagem:',
+            'Oorspronkelijke berichtkoppen:',
+        ],
+        'error'   => [
+            'Diagnostic information for administrators:',
+            'Diagnostische gegevens voor beheerders:',
+            'Error Details',
+            'stico para administradores:',
+        ],
+        'lhost'   => [
+            'Generating server: ',
+            'Bronserver: ',
+            'Servidor de origem: ',
+        ],
+        'message' => [
+            ' rejected your message to the following e',
+            'Delivery has failed to these recipients or groups:',
+            'Falha na entrega a estes destinat',
+            'Original Message Details',
+            'Uw bericht kan niet worden bezorgd bij de volgende geadresseerden of groepen:',
+        ],
+        'rfc3464' => ['Content-Type: message/delivery-status'],
     };
     state $statuslist = {
         # https://support.office.com/en-us/article/Email-non-delivery-reports-in-Office-365-51daa6b9-2e35-49c4-a0c9-df85bf8533c3
         qr/\A4[.]4[.]7\z/        => 'expired',
         qr/\A4[.]4[.]312\z/      => 'networkerror',
         qr/\A4[.]4[.]316\z/      => 'expired',
-        qr/\A4[.]7[.]26\z/       => 'securityerror',
+        qr/\A4[.]7[.]26\z/       => 'authfailure',
         qr/\A4[.]7[.][56]\d\d\z/ => 'blocked',
         qr/\A4[.]7[.]8[5-9]\d\z/ => 'blocked',
         qr/\A5[.]0[.]350\z/      => 'contenterror',
@@ -71,41 +91,37 @@ sub make {
         qr/\A5[.]7[.]1[23]\z/    => 'rejected',
         qr/\A5[.]7[.]124\z/      => 'rejected',
         qr/\A5[.]7[.]13[3-6]\z/  => 'rejected',
-        qr/\A5[.]7[.]23\z/       => 'blocked',
+        qr/\A5[.]7[.]23\z/       => 'authfailure',
         qr/\A5[.]7[.]25\z/       => 'networkerror',
         qr/\A5[.]7[.]50[1-3]\z/  => 'spamdetected',
         qr/\A5[.]7[.]50[4-5]\z/  => 'filtered',
         qr/\A5[.]7[.]50[6-7]\z/  => 'blocked',
         qr/\A5[.]7[.]508\z/      => 'toomanyconn',
-        qr/\A5[.]7[.]509\z/      => 'securityerror',
+        qr/\A5[.]7[.]509\z/      => 'authfailure',
         qr/\A5[.]7[.]510\z/      => 'notaccept',
         qr/\A5[.]7[.]511\z/      => 'rejected',
-        qr/\A5[.]7[.]512\z/      => 'securityerror',
+        qr/\A5[.]7[.]512\z/      => 'policyviolation',
         qr/\A5[.]7[.]57\z/       => 'securityerror',
         qr/\A5[.]7[.]60[6-9]\z/  => 'blocked',
         qr/\A5[.]7[.]6[1-4]\d\z/ => 'blocked',
         qr/\A5[.]7[.]7[0-4]\d\z/ => 'toomanyconn',
     };
-    state $recommands = {
-        'RCPT' => qr/unknown recipient or mailbox unavailable ->.+[<]?.+[@].+[.][a-zA-Z]+[>]?/,
-    };
 
-    require Sisimai::RFC1894;
     my $fieldtable = Sisimai::RFC1894->FIELDTABLE;
     my $permessage = {};    # (Hash) Store values of each Per-Message field
     my $dscontents = [__PACKAGE__->DELIVERYSTATUS];
-    my $emailsteak = Sisimai::RFC5322->fillet($mbody, $rebackbone);
+    my $emailparts = Sisimai::RFC5322->part($mbody, $boundaries);
     my $readcursor = 0;     # (Integer) Points the current cursor position
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
     my $endoferror = 0;     # (Integer) Flag for the end of error messages
     my $v = undef;
 
-    for my $e ( split("\n", $emailsteak->[0]) ) {
-        # Read error messages and delivery status lines from the head of the email
-        # to the previous line of the beginning of the original message.
+    for my $e ( split("\n", $emailparts->[0]) ) {
+        # Read error messages and delivery status lines from the head of the email to the previous
+        # line of the beginning of the original message.
         unless( $readcursor ) {
             # Beginning of the bounce message or message/delivery-status part
-            $readcursor |= $indicators->{'deliverystatus'} if $e =~ $markingsof->{'message'};
+            $readcursor |= $indicators->{'deliverystatus'} if grep { index($e, $_) > -1 } $startingof->{'message'}->@*;
             next;
         }
         next unless $readcursor & $indicators->{'deliverystatus'};
@@ -122,20 +138,24 @@ sub make {
         # Recipient Address:      kijitora@example.org
         # Subject:        Nyaan
         $v = $dscontents->[-1];
-        if( $e =~ /\A.+[@].+[<]mailto:(.+[@].+)[>]\z/ ||
-            $e =~ /\ARecipient[ ]Address:[ ]+(.+)\z/ ) {
+
+        my $p1 = index($e, '<mailto:');
+        my $p2 = index($e, 'Recipient Address: ');
+        if( $p1 > 1 || $p2 == 0 ) {
             # kijitora@example.com<mailto:kijitora@example.com>
+            # Recipient Address:      kijitora-nyaan@neko.kyoto.example.jp
             if( $v->{'recipient'} ) {
                 # There are multiple recipient addresses in the message body.
                 push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
                 $v = $dscontents->[-1];
             }
-            $v->{'recipient'} = $1;
+
+            $v->{'recipient'} = Sisimai::Address->s3s4(substr($e, index($e, ':') + 1,)); 
             $recipients++;
 
-        } elsif( $e =~ /\AGenerating server: (.+)\z/ ) {
+        } elsif( grep { index($e, $_) == 0 } $startingof->{'lhost'}->@* ) {
             # Generating server: FFFFFFFFFFFF.e0.prod.outlook.com
-            $permessage->{'lhost'} = lc $1;
+            $permessage->{'lhost'} = substr($e, index($e, ': ') + 2,);
 
         } else {
             if( $endoferror ) {
@@ -143,27 +163,43 @@ sub make {
                 next unless my $f = Sisimai::RFC1894->match($e);
                 next unless my $o = Sisimai::RFC1894->field($e);
                 next unless exists $fieldtable->{ $o->[0] };
-                next if $o->[0] =~ /\A(?:diagnostic-code|final-recipient)\z/;
-                $v->{ $fieldtable->{ $o->[0] } } = $o->[2];
 
-                next unless $f == 1;
-                $permessage->{ $fieldtable->{ $o->[0] } } = $o->[2];
+                if( $v->{'diagnosis'} ) {
+                    # Do not capture "Diagnostic-Code:" field because error message have already
+                    # been captured
+                    next if $o->[0] eq 'diagnostic-code' || $o->[0] eq 'final-recipient';
+                    $v->{ $fieldtable->{ $o->[0] } } = $o->[2];
 
+                    next unless $f == 1;
+                    $permessage->{ $fieldtable->{ $o->[0] } } = $o->[2];
+
+                } else {
+                    # Capture "Diagnostic-Code:" field because no error messages have been captured
+                    $v->{ $fieldtable->{ $o->[0] } } = $o->[2];
+                    $permessage->{ $fieldtable->{ $o->[0] } } = $o->[2];
+                }
             } else {
-                if( $e =~ $markingsof->{'error'} ) {
+                if( grep { index($e, $_) > -1 } $startingof->{'error'}->@* ) {
                     # Diagnostic information for administrators:
                     $v->{'diagnosis'} = $e;
+
                 } else {
                     # kijitora@example.com
                     # Remote Server returned '550 5.1.10 RESOLVER.ADR.RecipientNotFound; Recipien=
                     # t not found by SMTP address lookup'
-                    next unless $v->{'diagnosis'};
-                    if( $e =~ $markingsof->{'eoe'} ) {
-                        # Original message headers:
-                        $endoferror = 1;
-                        next;
+                    if( $v->{'diagnosis'} ) {
+                        # The error message text have already captured
+                        if( grep { index($e, $_) > -1 } $startingof->{'eoe'}->@* ) {
+                            # Original message headers:
+                            $endoferror = 1;
+                            next;
+                        }
+                        $v->{'diagnosis'} .= ' '.$e;
+
+                    } else {
+                        # The error message text has not been captured yet
+                        $endoferror = 1 if index($e, $startingof->{'rfc3464'}->[0]) == 0;
                     }
-                    $v->{'diagnosis'} .= ' '.$e;
                 }
             }
         }
@@ -180,9 +216,9 @@ sub make {
             $e->{'status'} = Sisimai::SMTP::Status->find($e->{'diagnosis'}) || $e->{'status'};
         }
 
-        for my $p ( keys %$recommands ) {
-            # Try to match with regular expressions defined in recommands
-            next unless $e->{'diagnosis'} =~ $recommands->{ $p };
+        for my $p ( keys %$commandset ) {
+            # Try to match with regular expressions defined in commandset
+            next unless Sisimai::String->aligned(\$e->{'diagnosis'}, $commandset->{ $p });
             $e->{'command'} = $p;
             last;
         }
@@ -196,7 +232,7 @@ sub make {
             last;
         }
     }
-    return { 'ds' => $dscontents, 'rfc822' => $emailsteak->[1] };
+    return { 'ds' => $dscontents, 'rfc822' => $emailparts->[1] };
 }
 
 1;
@@ -214,8 +250,8 @@ Sisimai::Lhost::Office365 - bounce mail parser class for Microsoft Office 365.
 
 =head1 DESCRIPTION
 
-Sisimai::Lhost::Office365 parses a bounce email which created by Microsoft
-Office 365. Methods in the module are called from only Sisimai::Message.
+Sisimai::Lhost::Office365 parses a bounce email which created by Microsoft Office 365. Methods in
+the module are called from only Sisimai::Message.
 
 =head1 CLASS METHODS
 
@@ -225,10 +261,10 @@ C<description()> returns description string of this module.
 
     print Sisimai::Lhost::Office365->description;
 
-=head2 C<B<make(I<header data>, I<reference to body string>)>>
+=head2 C<B<inquire(I<header data>, I<reference to body string>)>>
 
-C<make()> method parses a bounced email and return results as a array reference.
-See Sisimai::Message for more details.
+C<inquire()> method parses a bounced email and return results as a array reference. See Sisimai::Message
+for more details.
 
 =head1 AUTHOR
 
@@ -236,7 +272,7 @@ azumakuniyuki
 
 =head1 COPYRIGHT
 
-Copyright (C) 2016-2020 azumakuniyuki, All rights reserved.
+Copyright (C) 2016-2024 azumakuniyuki, All rights reserved.
 
 =head1 LICENSE
 
