@@ -5,22 +5,28 @@ use strict;
 use warnings;
 
 sub description { 'V8Sendmail: /usr/sbin/sendmail' }
-sub make {
+sub inquire {
     # Parse bounce messages from Sendmail
     # @param    [Hash] mhead    Message headers of a bounce email
     # @param    [String] mbody  Message body of a bounce email
     # @return   [Hash]          Bounce data list and message/rfc822 part
-    # @return   [Undef]         failed to parse or the arguments are missing
+    # @return   [undef]         failed to parse or the arguments are missing
     # @since v4.0.0
     my $class = shift;
     my $mhead = shift // return undef;
     my $mbody = shift // return undef;
+    my $match = 0;
 
-    return undef unless $mhead->{'subject'} =~ /(?:see transcript for details\z|\AWarning: )/;
     return undef if $mhead->{'x-aol-ip'};   # X-AOL-IP is a header defined in AOL
+    $match ||= 1 if index($mhead->{'subject'}, 'see transcript for details') > -1;
+    $match ||= 1 if index($mhead->{'subject'}, 'Warning: ')                  == 0;
+    return undef unless $match > 0;
 
+    require Sisimai::SMTP::Reply;
+    require Sisimai::SMTP::Status;
+    require Sisimai::SMTP::Command;
     state $indicators = __PACKAGE__->INDICATORS;
-    state $rebackbone = qr<^Content-Type:[ ](?:message/rfc822|text/rfc822-headers)>m;
+    state $boundaries = ['Content-Type: message/rfc822', 'Content-Type: text/rfc822-headers'];
     state $startingof = {
         #   savemail.c:1040|if (printheader && !putline("   ----- Transcript of session follows -----\n",
         #   savemail.c:1041|          mci))
@@ -33,24 +39,22 @@ sub make {
         'error'   => ['... while talking to '],
     };
 
-    require Sisimai::RFC1894;
     my $fieldtable = Sisimai::RFC1894->FIELDTABLE;
     my $permessage = {};    # (Hash) Store values of each Per-Message field
-
     my $dscontents = [__PACKAGE__->DELIVERYSTATUS];
-    my $emailsteak = Sisimai::RFC5322->fillet($mbody, $rebackbone);
+    my $emailparts = Sisimai::RFC5322->part($mbody, $boundaries);
     my $readcursor = 0;     # (Integer) Points the current cursor position
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
-    my $commandtxt = '';    # (String) SMTP Command name begin with the string '>>>'
+    my $thecommand = '';    # (String) SMTP Command name begin with the string '>>>'
     my $esmtpreply = [];    # (Array) Reply from remote server on SMTP session
     my $sessionerr = 0;     # (Integer) Flag, 1 if it is SMTP session error
     my $anotherset = {};    # (Hash) Another error information
     my $v = undef;
     my $p = '';
 
-    for my $e ( split("\n", $emailsteak->[0]) ) {
-        # Read error messages and delivery status lines from the head of the email
-        # to the previous line of the beginning of the original message.
+    for my $e ( split("\n", $emailparts->[0]) ) {
+        # Read error messages and delivery status lines from the head of the email to the previous
+        # line of the beginning of the original message.
         unless( $readcursor ) {
             # Beginning of the bounce message or message/delivery-status part
             $readcursor |= $indicators->{'deliverystatus'} if index($e, $startingof->{'message'}->[0]) == 0;
@@ -104,13 +108,14 @@ sub make {
             # 554 5.0.0 Service unavailable
             if( substr($e, 0, 1) ne ' ') {
                 # Other error messages
-                if( $e =~ /\A[>]{3}[ ]+([A-Z]{4})[ ]?/ ) {
+                if( index($e, '>>> ') == 0 ) {
                     # >>> DATA
-                    $commandtxt = $1;
+                    $thecommand = Sisimai::SMTP::Command->find($e);
 
-                } elsif( $e =~ /\A[<]{3}[ ]+(.+)\z/ ) {
+                } elsif( index($e, '<<< ') == 0 ) {
                     # <<< Response
-                    push @$esmtpreply, $1 unless grep { $1 eq $_ } @$esmtpreply;
+                    my $cv = substr($e, 4,);
+                    push @$esmtpreply, $cv unless grep { $cv eq $_ } @$esmtpreply;
 
                 } else {
                     # Detect SMTP session error or connection error
@@ -122,21 +127,23 @@ sub make {
                         next;
                     }
 
-                    if( $e =~ /\A[<](.+)[>][.]+ (.+)\z/ ) {
+                    if( index($e, '<') == 0 && Sisimai::String->aligned(\$e, ['@', '>.', ' ']) ) {
                         # <kijitora@example.co.jp>... Deferred: Name server: example.co.jp.: host name lookup failure
-                        $anotherset->{'recipient'} = $1;
-                        $anotherset->{'diagnosis'} = $2;
+                        $anotherset->{'recipient'} = Sisimai::Address->s3s4(substr($e, 0, index($e, '>')));
+                        $anotherset->{'diagnosis'} = substr($e, index($e, ' ') + 1,);
 
                     } else {
                         # ----- Transcript of session follows -----
                         # Message could not be delivered for too long
                         # Message will be deleted from queue
-                        if( $e =~ /\A[45]\d\d[ \t]([45][.]\d[.]\d)[ \t].+/ ) {
+                        my $cr = Sisimai::SMTP::Reply->find($e)  || '';
+                        my $cs = Sisimai::SMTP::Status->find($e) || '';
+                        if( length($cr.$cs) > 7 ) {
                             # 550 5.1.2 <kijitora@example.org>... Message
                             #
                             # DBI connect('dbname=...')
                             # 554 5.3.0 unknown mailer error 255
-                            $anotherset->{'status'} = $1;
+                            $anotherset->{'status'}     = $cs;
                             $anotherset->{'diagnosis'} .= ' '.$e;
 
                         } elsif( index($e, 'Message: ') == 0 || index($e, 'Warning: ') == 0 ) {
@@ -149,8 +156,8 @@ sub make {
             } else {
                 # Continued line of the value of Diagnostic-Code field
                 next unless index($p, 'Diagnostic-Code:') == 0;
-                next unless $e =~ /\A[ \t]+(.+)\z/;
-                $v->{'diagnosis'} .= ' '.$1;
+                next unless index($e, ' ') == 0;
+                $v->{'diagnosis'} .= ' '.Sisimai::String->sweep($e);
             }
         }
     } continue {
@@ -161,38 +168,45 @@ sub make {
 
     for my $e ( @$dscontents ) {
         # Set default values if each value is empty.
-        $e->{'lhost'}   ||= $permessage->{'rhost'};
-        $e->{ $_ }      ||= $permessage->{ $_ } || '' for keys %$permessage;
-        $e->{'command'} ||= $commandtxt         || '';
-        $e->{'command'} ||= 'EHLO' if scalar @$esmtpreply;
+        $e->{ $_ } ||= $permessage->{ $_ } || '' for keys %$permessage;
 
         if( exists $anotherset->{'diagnosis'} && $anotherset->{'diagnosis'} ) {
             # Copy alternative error message
-            $e->{'diagnosis'}   = $anotherset->{'diagnosis'} if $e->{'diagnosis'} =~ /\A[ \t]+\z/;
-            $e->{'diagnosis'} ||= $anotherset->{'diagnosis'};
+            $e->{'diagnosis'}   = $anotherset->{'diagnosis'} if index($e->{'diagnosis'}, ' ') == 0;
             $e->{'diagnosis'}   = $anotherset->{'diagnosis'} if $e->{'diagnosis'} =~ /\A\d+\z/;
+            $e->{'diagnosis'} ||= $anotherset->{'diagnosis'};
         }
-        if( scalar @$esmtpreply ) {
-            # Replace the error message in "diagnosis" with the ESMTP Reply
-            my $r = join(' ', @$esmtpreply);
-            $e->{'diagnosis'} = $r if length($r) > length($e->{'diagnosis'});
+
+        while(1) {
+            # Replace or append the error message in "diagnosis" with the ESMTP Reply Code when the
+            # following conditions have matched
+            last unless scalar @$esmtpreply;
+            last unless $recipients == 1;
+
+            $e->{'diagnosis'} = sprintf("%s %s", join(' ', @$esmtpreply), $e->{'diagnosis'});
+            last;
         }
         $e->{'diagnosis'} = Sisimai::String->sweep($e->{'diagnosis'});
+        $e->{'command'} ||= $thecommand || Sisimai::SMTP::Command->find($e->{'diagnosis'}) || '';
+        $e->{'command'} ||= 'EHLO' if scalar @$esmtpreply;
 
-        if( exists $anotherset->{'status'} && $anotherset->{'status'} ) {
-            # Check alternative status code
-            if( ! $e->{'status'} || $e->{'status'} !~ /\A[45][.]\d[.]\d{1,3}\z/ ) {
-                # Override alternative status code
-                $e->{'status'} = $anotherset->{'status'};
-            }
+        while(1) {
+            # Check alternative status code and override it
+            last unless exists $anotherset->{'status'};
+            last unless length $anotherset->{'status'};
+            last if     Sisimai::SMTP::Status->test($e->{'status'});
+
+            $e->{'status'} = $anotherset->{'status'};
+            last;
         }
 
         # @example.jp, no local part
-        # Get email address from the value of Diagnostic-Code header
-        next if $e->{'recipient'} =~ /\A[^ ]+[@][^ ]+\z/;
-        $e->{'recipient'} = $1 if $e->{'diagnosis'} =~ /[<]([^ ]+[@][^ ]+)[>]/;
+        # Get email address from the value of Diagnostic-Code field
+        next unless index($e->{'recipient'}, '@') == 0;
+        my $cv = Sisimai::Address->find($e->{'diagnosis'}, 1) || [];
+        $e->{'recipient'} = $cv->[0]->{'address'} if scalar @$cv;
     }
-    return { 'ds' => $dscontents, 'rfc822' => $emailsteak->[1] };
+    return { 'ds' => $dscontents, 'rfc822' => $emailparts->[1] };
 }
 
 1;
@@ -210,8 +224,8 @@ Sisimai::Lhost::Sendmail - bounce mail parser class for v8 Sendmail.
 
 =head1 DESCRIPTION
 
-Sisimai::Lhost::Sendmail parses a bounce email which created by v8 Sendmail.
-Methods in the module are called from only Sisimai::Message.
+Sisimai::Lhost::Sendmail parses a bounce email which created by v8 Sendmail. Methods in the module
+are called from only Sisimai::Message.
 
 =head1 CLASS METHODS
 
@@ -221,10 +235,10 @@ C<description()> returns description string of this module.
 
     print Sisimai::Lhost::Sendmail->description;
 
-=head2 C<B<make(I<header data>, I<reference to body string>)>>
+=head2 C<B<inquire(I<header data>, I<reference to body string>)>>
 
-C<make()> method parses a bounced email and return results as a array reference.
-See Sisimai::Message for more details.
+C<inquire()> method parses a bounced email and return results as a array reference. See Sisimai::Message
+for more details.
 
 =head1 AUTHOR
 
@@ -232,7 +246,7 @@ azumakuniyuki
 
 =head1 COPYRIGHT
 
-Copyright (C) 2014-2021 azumakuniyuki, All rights reserved.
+Copyright (C) 2014-2024 azumakuniyuki, All rights reserved.
 
 =head1 LICENSE
 
