@@ -24,16 +24,15 @@ Sisimai::Reason::Delivered - Email delivered successfully
 
 =head1 DESCRIPTION
 
-Sisimai::Reason::Delivered checks the email you sent is delivered successfully
-or not by matching diagnostic messages with message patterns. Sisimai will set
-"delivered" to the value of "reason" when Status: field in the bounce message
-begins with "2" like following:
+Sisimai::Reason::Delivered checks the email you sent is delivered successfully or not by matching
+diagnostic messages with message patterns. Sisimai will set "delivered" to the value of "reason"
+when Status: field in the bounce message begins with "2" like following:
 
     Final-Recipient: rfc822; kijitora@neko.nyaan.jp
-    Action: deliverable
+    Action: delivered
     Status: 2.1.5
     Diagnostic-Code: SMTP; 250 2.1.5 OK
-    
+
 This class is called only Sisimai->reason method. This is NOT AN ERROR reason.
 
 =head1 CLASS METHODS
@@ -48,7 +47,7 @@ C<text()> returns string: C<delivered>.
 
 C<match()> always return undef
 
-=head2 C<B<true(I<Sisimai::Data>)>>
+=head2 C<B<true(I<Sisimai::Fact>)>>
 
 C<true()> always return undef
 
@@ -58,7 +57,7 @@ azumakuniyuki
 
 =head1 COPYRIGHT
 
-Copyright (C) 2016 azumakuniyuki, All rights reserved.
+Copyright (C) 2016,2020,2021 azumakuniyuki, All rights reserved.
 
 =head1 LICENSE
 
