@@ -16,7 +16,7 @@ sub match {
 
 sub true  {
     # On hold, Could not decide the bounce reason...
-    # @param    [Sisimai::Data] argvs   Object to be detected the reason
+    # @param    [Sisimai::Fact] argvs   Object to be detected the reason
     # @return   [Integer]               1: Status code is "onhold"
     #                                   0: is not "onhold"
     # @since v4.1.28
@@ -24,9 +24,9 @@ sub true  {
     my $class = shift;
     my $argvs = shift // return undef;
 
-    return undef unless $argvs->deliverystatus;
-    return 1 if $argvs->reason eq 'onhold';
-    return 1 if (Sisimai::SMTP::Status->name($argvs->deliverystatus) || '') eq 'onhold';
+    return undef unless $argvs->{'deliverystatus'};
+    return 1 if $argvs->{'reason'} eq 'onhold';
+    return 1 if (Sisimai::SMTP::Status->name($argvs->{'deliverystatus'}) || '') eq 'onhold';
     return 0
 }
 
@@ -46,10 +46,8 @@ Sisimai::Reason::OnHold - Bounce reason is C<onhold> or not.
 
 =head1 DESCRIPTION
 
-Sisimai::Reason::OnHold checks the bounce reason is C<onhold> or not. This class
-is called only Sisimai::Reason class.
-
-Sisimai will set C<onhold> to the reason of email bounce if there is no (or
+Sisimai::Reason::OnHold checks the bounce reason is C<onhold> or not. This class is called only
+Sisimai::Reason class. Sisimai will set C<onhold> to the reason of email bounce if there is no (or
 less) detailed information about email bounce for judging the reason.
 
 =head1 CLASS METHODS
@@ -66,10 +64,10 @@ C<match()> returns 1 if the argument matched with patterns defined in this class
 
     print Sisimai::Reason::OnHold->match; # 0;
 
-=head2 C<B<true(I<Sisimai::Data>)>>
+=head2 C<B<true(I<Sisimai::Fact>)>>
 
-C<true()> returns 1 if the bounce reason is C<onhold>. The argument must be
-Sisimai::Data object and this method is called only from Sisimai::Reason class.
+C<true()> returns 1 if the bounce reason is C<onhold>. The argument must be Sisimai::Fact object
+and this method is called only from Sisimai::Reason class.
 
 =head1 AUTHOR
 
@@ -77,7 +75,7 @@ azumakuniyuki
 
 =head1 COPYRIGHT
 
-Copyright (C) 2014-2016,2018 azumakuniyuki, All rights reserved.
+Copyright (C) 2014-2016,2018,2020,2021 azumakuniyuki, All rights reserved.
 
 =head1 LICENSE
 
