@@ -24,7 +24,7 @@ sub match {
 
 sub true {
     # The bounce reason is system full or not
-    # @param    [Sisimai::Data] argvs   Object to be detected the reason
+    # @param    [Sisimai::Fact] argvs   Object to be detected the reason
     # @return   [Integer]               1: is system full
     #                                   0: is not system full
     # @see http://www.ietf.org/rfc/rfc2822.txt
@@ -47,12 +47,11 @@ Sisimai::Reason::SystemFull - Bounce reason is C<systemfull> or not.
 
 =head1 DESCRIPTION
 
-Sisimai::Reason::SystemFull checks the bounce reason is C<systemfull> or not.
-This class is called only Sisimai::Reason class.
+Sisimai::Reason::SystemFull checks the bounce reason is C<systemfull> or not. This class is called
+only Sisimai::Reason class.
 
-This is the error that a destination mail server's disk (or spool) is full.
-Sisimai will set C<systemfull> to the reason of email bounce if the value of
-Status: field in a bounce email is C<4.3.1> or C<5.3.1>.
+This is the error that a destination mail server's disk (or spool) is full. Sisimai will set C<systemfull>
+to the reason of email bounce if the value of Status: field in a bounce email is C<4.3.1> or C<5.3.1>.
 
 =head1 CLASS METHODS
 
@@ -68,10 +67,10 @@ C<match()> returns 1 if the argument matched with patterns defined in this class
 
     print Sisimai::Reason::SystemFull->match('Mail System Full');   # 1
 
-=head2 C<B<true(I<Sisimai::Data>)>>
+=head2 C<B<true(I<Sisimai::Fact>)>>
 
-C<true()> returns 1 if the bounce reason is C<systemfull>. The argument must be
-Sisimai::Data object and this method is called only from Sisimai::Reason class.
+C<true()> returns 1 if the bounce reason is C<systemfull>. The argument must be Sisimai::Fact object
+and this method is called only from Sisimai::Reason class.
 
 =head1 AUTHOR
 
@@ -79,7 +78,7 @@ azumakuniyuki
 
 =head1 COPYRIGHT
 
-Copyright (C) 2014-2016,2018 azumakuniyuki, All rights reserved.
+Copyright (C) 2014-2016,2018,2020,2021 azumakuniyuki, All rights reserved.
 
 =head1 LICENSE
 
